
var ServiceCredentials = function(aId){	
	this.iUrl = window.location.protocol + "//" + window.location.host + "/" + aId + "/Credentials/control";
	this.iDomain = "av.openhome.org";
	if (this.iDomain == "upnp.org") {
		this.iDomain = "schemas.upnp.org";
    }
	this.iDomain = this.iDomain.replace(/\./,"-");
	this.iType = "Credentials";
	this.iVersion = "1";
	
	this.iVariables = {};
		this.iVariables["Ids"] = new ServiceVariable("Ids");
		this.iVariables["PublicKey"] = new ServiceVariable("PublicKey");
		this.iVariables["SequenceNumber"] = new ServiceVariable("SequenceNumber");
		this.iVariables["String"] = new ServiceVariable("String");
		this.iVariables["Binary"] = new ServiceVariable("Binary");
		this.iVariables["Bool"] = new ServiceVariable("Bool");
}


ServiceCredentials.prototype.ServiceName = function(){
	return this.iType;
}

ServiceCredentials.prototype.Variables = function(){
	return this.iVariables;
}

ServiceCredentials.prototype.VariableNames = function(){
	var result = [];
	for (var variable in this.iVariables){
		if (this.iVariables.hasOwnProperty(variable)){
			result[result.length] = variable;
		}
	}
	return result;
}


ServiceCredentials.prototype.Set = function(Id, UserName, Password, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Set", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteStringParameter("Id", Id);
    request.WriteStringParameter("UserName", UserName);
    request.WriteBinaryParameter("Password", Password);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceCredentials.prototype.Clear = function(Id, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Clear", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteStringParameter("Id", Id);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceCredentials.prototype.SetEnabled = function(Id, Enabled, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("SetEnabled", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteStringParameter("Id", Id);
    request.WriteBoolParameter("Enabled", Enabled);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceCredentials.prototype.Get = function(Id, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Get", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteStringParameter("Id", Id);
    request.Send(function(result){
		result["UserName"] = request.ReadStringParameter(result["UserName"]);	
		result["Password"] = request.ReadBinaryParameter(result["Password"]);	
		result["Enabled"] = request.ReadBoolParameter(result["Enabled"]);	
		result["Status"] = request.ReadStringParameter(result["Status"]);	
		result["Data"] = request.ReadStringParameter(result["Data"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceCredentials.prototype.Login = function(Id, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Login", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteStringParameter("Id", Id);
    request.Send(function(result){
		result["Token"] = request.ReadStringParameter(result["Token"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceCredentials.prototype.ReLogin = function(Id, CurrentToken, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("ReLogin", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteStringParameter("Id", Id);
    request.WriteStringParameter("CurrentToken", CurrentToken);
    request.Send(function(result){
		result["NewToken"] = request.ReadStringParameter(result["NewToken"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceCredentials.prototype.GetIds = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("GetIds", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Ids"] = request.ReadStringParameter(result["Ids"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceCredentials.prototype.GetPublicKey = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("GetPublicKey", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["PublicKey"] = request.ReadStringParameter(result["PublicKey"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceCredentials.prototype.GetSequenceNumber = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("GetSequenceNumber", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["SequenceNumber"] = request.ReadIntParameter(result["SequenceNumber"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

