
var ServiceFlash = function(aId){	
	this.iUrl = window.location.protocol + "//" + window.location.host + "/" + aId + "/Flash/control";
	this.iDomain = "linn.co.uk";
	if (this.iDomain == "upnp.org") {
		this.iDomain = "schemas.upnp.org";
    }
	this.iDomain = this.iDomain.replace(/\./,"-");
	this.iType = "Flash";
	this.iVersion = "1";
	
	this.iVariables = {};
		this.iVariables["FlashId"] = new ServiceVariable("FlashId");
		this.iVariables["Address"] = new ServiceVariable("Address");
		this.iVariables["Index"] = new ServiceVariable("Index");
		this.iVariables["Count"] = new ServiceVariable("Count");
		this.iVariables["Bytes"] = new ServiceVariable("Bytes");
		this.iVariables["Data"] = new ServiceVariable("Data");
}


ServiceFlash.prototype.ServiceName = function(){
	return this.iType;
}

ServiceFlash.prototype.Variables = function(){
	return this.iVariables;
}

ServiceFlash.prototype.VariableNames = function(){
	var result = [];
	for (var variable in this.iVariables){
		if (this.iVariables.hasOwnProperty(variable)){
			result[result.length] = variable;
		}
	}
	return result;
}


ServiceFlash.prototype.Read = function(aId, aAddress, aLength, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Read", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("aId", aId);
    request.WriteIntParameter("aAddress", aAddress);
    request.WriteIntParameter("aLength", aLength);
    request.Send(function(result){
		result["aBuffer"] = request.ReadBinaryParameter(result["aBuffer"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceFlash.prototype.Write = function(aId, aAddress, aSource, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Write", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("aId", aId);
    request.WriteIntParameter("aAddress", aAddress);
    request.WriteBinaryParameter("aSource", aSource);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceFlash.prototype.Erase = function(aId, aAddress, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Erase", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("aId", aId);
    request.WriteIntParameter("aAddress", aAddress);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceFlash.prototype.EraseSector = function(aId, aSector, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("EraseSector", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("aId", aId);
    request.WriteIntParameter("aSector", aSector);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceFlash.prototype.EraseSectors = function(aId, aFirstSector, aLastSector, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("EraseSectors", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("aId", aId);
    request.WriteIntParameter("aFirstSector", aFirstSector);
    request.WriteIntParameter("aLastSector", aLastSector);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceFlash.prototype.EraseChip = function(aId, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("EraseChip", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("aId", aId);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceFlash.prototype.Sectors = function(aId, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Sectors", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("aId", aId);
    request.Send(function(result){
		result["aSectors"] = request.ReadIntParameter(result["aSectors"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceFlash.prototype.SectorBytes = function(aId, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("SectorBytes", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("aId", aId);
    request.Send(function(result){
		result["aBytes"] = request.ReadIntParameter(result["aBytes"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceFlash.prototype.RomDirInfo = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("RomDirInfo", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["aFlashIdMain"] = request.ReadIntParameter(result["aFlashIdMain"]);	
		result["aOffsetMain"] = request.ReadIntParameter(result["aOffsetMain"]);	
		result["aBytesMain"] = request.ReadIntParameter(result["aBytesMain"]);	
		result["aFlashIdFallback"] = request.ReadIntParameter(result["aFlashIdFallback"]);	
		result["aOffsetFallback"] = request.ReadIntParameter(result["aOffsetFallback"]);	
		result["aBytesFallback"] = request.ReadIntParameter(result["aBytesFallback"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

