
var ServiceInfo = function(aId){	
	this.iUrl = window.location.protocol + "//" + window.location.host + "/" + aId + "/Info/control";
	this.iDomain = "av.openhome.org";
	if (this.iDomain == "upnp.org") {
		this.iDomain = "schemas.upnp.org";
    }
	this.iDomain = this.iDomain.replace(/\./,"-");
	this.iType = "Info";
	this.iVersion = "1";
	
	this.iVariables = {};
		this.iVariables["TrackCount"] = new ServiceVariable("TrackCount");
		this.iVariables["DetailsCount"] = new ServiceVariable("DetailsCount");
		this.iVariables["MetatextCount"] = new ServiceVariable("MetatextCount");
		this.iVariables["Uri"] = new ServiceVariable("Uri");
		this.iVariables["Metadata"] = new ServiceVariable("Metadata");
		this.iVariables["Duration"] = new ServiceVariable("Duration");
		this.iVariables["BitRate"] = new ServiceVariable("BitRate");
		this.iVariables["BitDepth"] = new ServiceVariable("BitDepth");
		this.iVariables["SampleRate"] = new ServiceVariable("SampleRate");
		this.iVariables["Lossless"] = new ServiceVariable("Lossless");
		this.iVariables["CodecName"] = new ServiceVariable("CodecName");
		this.iVariables["Metatext"] = new ServiceVariable("Metatext");
}


ServiceInfo.prototype.ServiceName = function(){
	return this.iType;
}

ServiceInfo.prototype.Variables = function(){
	return this.iVariables;
}

ServiceInfo.prototype.VariableNames = function(){
	var result = [];
	for (var variable in this.iVariables){
		if (this.iVariables.hasOwnProperty(variable)){
			result[result.length] = variable;
		}
	}
	return result;
}


ServiceInfo.prototype.Counters = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Counters", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["TrackCount"] = request.ReadIntParameter(result["TrackCount"]);	
		result["DetailsCount"] = request.ReadIntParameter(result["DetailsCount"]);	
		result["MetatextCount"] = request.ReadIntParameter(result["MetatextCount"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceInfo.prototype.Track = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Track", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Uri"] = request.ReadStringParameter(result["Uri"]);	
		result["Metadata"] = request.ReadStringParameter(result["Metadata"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceInfo.prototype.Details = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Details", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Duration"] = request.ReadIntParameter(result["Duration"]);	
		result["BitRate"] = request.ReadIntParameter(result["BitRate"]);	
		result["BitDepth"] = request.ReadIntParameter(result["BitDepth"]);	
		result["SampleRate"] = request.ReadIntParameter(result["SampleRate"]);	
		result["Lossless"] = request.ReadBoolParameter(result["Lossless"]);	
		result["CodecName"] = request.ReadStringParameter(result["CodecName"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceInfo.prototype.Metatext = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Metatext", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Value"] = request.ReadStringParameter(result["Value"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

