
var ServiceJukebox = function(aId){	
	this.iUrl = window.location.protocol + "//" + window.location.host + "/" + aId + "/Jukebox/control";
	this.iDomain = "linn.co.uk";
	if (this.iDomain == "upnp.org") {
		this.iDomain = "schemas.upnp.org";
    }
	this.iDomain = this.iDomain.replace(/\./,"-");
	this.iType = "Jukebox";
	this.iVersion = "3";
	
	this.iVariables = {};
		this.iVariables["Metadata"] = new ServiceVariable("Metadata");
		this.iVariables["CurrentPreset"] = new ServiceVariable("CurrentPreset");
		this.iVariables["CurrentBookmark"] = new ServiceVariable("CurrentBookmark");
}


ServiceJukebox.prototype.ServiceName = function(){
	return this.iType;
}

ServiceJukebox.prototype.Variables = function(){
	return this.iVariables;
}

ServiceJukebox.prototype.VariableNames = function(){
	var result = [];
	for (var variable in this.iVariables){
		if (this.iVariables.hasOwnProperty(variable)){
			result[result.length] = variable;
		}
	}
	return result;
}


ServiceJukebox.prototype.SetCurrentPreset = function(aPreset, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("SetCurrentPreset", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("aPreset", aPreset);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceJukebox.prototype.CurrentPreset = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("CurrentPreset", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["aPreset"] = request.ReadIntParameter(result["aPreset"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceJukebox.prototype.SetCurrentBookmark = function(aBookmark, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("SetCurrentBookmark", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("aBookmark", aBookmark);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceJukebox.prototype.CurrentBookmark = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("CurrentBookmark", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["aBookmark"] = request.ReadIntParameter(result["aBookmark"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceJukebox.prototype.PresetMetadata = function(aPreset, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("PresetMetadata", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("aPreset", aPreset);
    request.Send(function(result){
		result["aMetadata"] = request.ReadStringParameter(result["aMetadata"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceJukebox.prototype.BookmarkMetadata = function(aBookmark, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("BookmarkMetadata", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("aBookmark", aBookmark);
    request.Send(function(result){
		result["aMetadata"] = request.ReadStringParameter(result["aMetadata"]);	
		result["aFirstPreset"] = request.ReadIntParameter(result["aFirstPreset"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceJukebox.prototype.PresetMetadataList = function(aStartPreset, aEndPreset, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("PresetMetadataList", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("aStartPreset", aStartPreset);
    request.WriteIntParameter("aEndPreset", aEndPreset);
    request.Send(function(result){
		result["aMetadataList"] = request.ReadStringParameter(result["aMetadataList"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceJukebox.prototype.BookmarkMetadataList = function(aStartBookmark, aEndBookmark, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("BookmarkMetadataList", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("aStartBookmark", aStartBookmark);
    request.WriteIntParameter("aEndBookmark", aEndBookmark);
    request.Send(function(result){
		result["aMetadataList"] = request.ReadStringParameter(result["aMetadataList"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceJukebox.prototype.LoadManifestFile = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("LoadManifestFile", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["aTotalPresets"] = request.ReadIntParameter(result["aTotalPresets"]);	
		result["aTotalBookmarks"] = request.ReadIntParameter(result["aTotalBookmarks"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

