
var ServicePlaylist = function(aId){	
	this.iUrl = window.location.protocol + "//" + window.location.host + "/" + aId + "/Playlist/control";
	this.iDomain = "av.openhome.org";
	if (this.iDomain == "upnp.org") {
		this.iDomain = "schemas.upnp.org";
    }
	this.iDomain = this.iDomain.replace(/\./,"-");
	this.iType = "Playlist";
	this.iVersion = "1";
	
	this.iVariables = {};
		this.iVariables["TransportState"] = new ServiceVariable("TransportState");
		this.iVariables["Repeat"] = new ServiceVariable("Repeat");
		this.iVariables["Shuffle"] = new ServiceVariable("Shuffle");
		this.iVariables["Id"] = new ServiceVariable("Id");
		this.iVariables["IdArray"] = new ServiceVariable("IdArray");
		this.iVariables["TracksMax"] = new ServiceVariable("TracksMax");
		this.iVariables["ProtocolInfo"] = new ServiceVariable("ProtocolInfo");
		this.iVariables["Index"] = new ServiceVariable("Index");
		this.iVariables["Relative"] = new ServiceVariable("Relative");
		this.iVariables["Absolute"] = new ServiceVariable("Absolute");
		this.iVariables["IdList"] = new ServiceVariable("IdList");
		this.iVariables["TrackList"] = new ServiceVariable("TrackList");
		this.iVariables["Uri"] = new ServiceVariable("Uri");
		this.iVariables["Metadata"] = new ServiceVariable("Metadata");
		this.iVariables["IdArrayToken"] = new ServiceVariable("IdArrayToken");
		this.iVariables["IdArrayChanged"] = new ServiceVariable("IdArrayChanged");
}

ServicePlaylist.kTransportStatePlaying = "Playing";
ServicePlaylist.kTransportStatePaused = "Paused";
ServicePlaylist.kTransportStateStopped = "Stopped";
ServicePlaylist.kTransportStateBuffering = "Buffering";

ServicePlaylist.prototype.ServiceName = function(){
	return this.iType;
}

ServicePlaylist.prototype.Variables = function(){
	return this.iVariables;
}

ServicePlaylist.prototype.VariableNames = function(){
	var result = [];
	for (var variable in this.iVariables){
		if (this.iVariables.hasOwnProperty(variable)){
			result[result.length] = variable;
		}
	}
	return result;
}


ServicePlaylist.prototype.Play = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Play", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylist.prototype.Pause = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Pause", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylist.prototype.Stop = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Stop", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylist.prototype.Next = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Next", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylist.prototype.Previous = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Previous", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylist.prototype.SetRepeat = function(Value, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("SetRepeat", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteBoolParameter("Value", Value);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylist.prototype.Repeat = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Repeat", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Value"] = request.ReadBoolParameter(result["Value"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylist.prototype.SetShuffle = function(Value, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("SetShuffle", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteBoolParameter("Value", Value);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylist.prototype.Shuffle = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Shuffle", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Value"] = request.ReadBoolParameter(result["Value"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylist.prototype.SeekSecondAbsolute = function(Value, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("SeekSecondAbsolute", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("Value", Value);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylist.prototype.SeekSecondRelative = function(Value, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("SeekSecondRelative", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("Value", Value);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylist.prototype.SeekId = function(Value, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("SeekId", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("Value", Value);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylist.prototype.SeekIndex = function(Value, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("SeekIndex", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("Value", Value);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylist.prototype.TransportState = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("TransportState", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Value"] = request.ReadStringParameter(result["Value"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylist.prototype.Id = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Id", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Value"] = request.ReadIntParameter(result["Value"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylist.prototype.Read = function(Id, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Read", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("Id", Id);
    request.Send(function(result){
		result["Uri"] = request.ReadStringParameter(result["Uri"]);	
		result["Metadata"] = request.ReadStringParameter(result["Metadata"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylist.prototype.ReadList = function(IdList, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("ReadList", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteStringParameter("IdList", IdList);
    request.Send(function(result){
		result["TrackList"] = request.ReadStringParameter(result["TrackList"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylist.prototype.Insert = function(AfterId, Uri, Metadata, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Insert", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("AfterId", AfterId);
    request.WriteStringParameter("Uri", Uri);
    request.WriteStringParameter("Metadata", Metadata);
    request.Send(function(result){
		result["NewId"] = request.ReadIntParameter(result["NewId"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylist.prototype.DeleteId = function(Value, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("DeleteId", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("Value", Value);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylist.prototype.DeleteAll = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("DeleteAll", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylist.prototype.TracksMax = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("TracksMax", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Value"] = request.ReadIntParameter(result["Value"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylist.prototype.IdArray = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("IdArray", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Token"] = request.ReadIntParameter(result["Token"]);	
		result["Array"] = request.ReadBinaryParameter(result["Array"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylist.prototype.IdArrayChanged = function(Token, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("IdArrayChanged", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("Token", Token);
    request.Send(function(result){
		result["Value"] = request.ReadBoolParameter(result["Value"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylist.prototype.ProtocolInfo = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("ProtocolInfo", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Value"] = request.ReadStringParameter(result["Value"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

