
var ServicePlaylistManager = function(aId){	
	this.iUrl = window.location.protocol + "//" + window.location.host + "/" + aId + "/PlaylistManager/control";
	this.iDomain = "av.openhome.org";
	if (this.iDomain == "upnp.org") {
		this.iDomain = "schemas.upnp.org";
    }
	this.iDomain = this.iDomain.replace(/\./,"-");
	this.iType = "PlaylistManager";
	this.iVersion = "1";
	
	this.iVariables = {};
		this.iVariables["Metadata"] = new ServiceVariable("Metadata");
		this.iVariables["ImagesXml"] = new ServiceVariable("ImagesXml");
		this.iVariables["IdArray"] = new ServiceVariable("IdArray");
		this.iVariables["TokenArray"] = new ServiceVariable("TokenArray");
		this.iVariables["PlaylistsMax"] = new ServiceVariable("PlaylistsMax");
		this.iVariables["TracksMax"] = new ServiceVariable("TracksMax");
		this.iVariables["PlaylistList"] = new ServiceVariable("PlaylistList");
		this.iVariables["Id"] = new ServiceVariable("Id");
		this.iVariables["IdList"] = new ServiceVariable("IdList");
		this.iVariables["Name"] = new ServiceVariable("Name");
		this.iVariables["Description"] = new ServiceVariable("Description");
		this.iVariables["ImageId"] = new ServiceVariable("ImageId");
		this.iVariables["TrackList"] = new ServiceVariable("TrackList");
		this.iVariables["ArraysToken"] = new ServiceVariable("ArraysToken");
		this.iVariables["IdArrayChanged"] = new ServiceVariable("IdArrayChanged");
}


ServicePlaylistManager.prototype.ServiceName = function(){
	return this.iType;
}

ServicePlaylistManager.prototype.Variables = function(){
	return this.iVariables;
}

ServicePlaylistManager.prototype.VariableNames = function(){
	var result = [];
	for (var variable in this.iVariables){
		if (this.iVariables.hasOwnProperty(variable)){
			result[result.length] = variable;
		}
	}
	return result;
}


ServicePlaylistManager.prototype.Metadata = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Metadata", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Metadata"] = request.ReadStringParameter(result["Metadata"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylistManager.prototype.ImagesXml = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("ImagesXml", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["ImagesXml"] = request.ReadStringParameter(result["ImagesXml"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylistManager.prototype.PlaylistReadArray = function(Id, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("PlaylistReadArray", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("Id", Id);
    request.Send(function(result){
		result["Array"] = request.ReadBinaryParameter(result["Array"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylistManager.prototype.PlaylistReadList = function(IdList, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("PlaylistReadList", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteStringParameter("IdList", IdList);
    request.Send(function(result){
		result["PlaylistList"] = request.ReadStringParameter(result["PlaylistList"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylistManager.prototype.PlaylistRead = function(Id, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("PlaylistRead", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("Id", Id);
    request.Send(function(result){
		result["Name"] = request.ReadStringParameter(result["Name"]);	
		result["Description"] = request.ReadStringParameter(result["Description"]);	
		result["ImageId"] = request.ReadIntParameter(result["ImageId"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylistManager.prototype.PlaylistSetName = function(Id, Name, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("PlaylistSetName", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("Id", Id);
    request.WriteStringParameter("Name", Name);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylistManager.prototype.PlaylistSetDescription = function(Id, Description, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("PlaylistSetDescription", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("Id", Id);
    request.WriteStringParameter("Description", Description);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylistManager.prototype.PlaylistSetImageId = function(Id, ImageId, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("PlaylistSetImageId", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("Id", Id);
    request.WriteIntParameter("ImageId", ImageId);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylistManager.prototype.PlaylistInsert = function(AfterId, Name, Description, ImageId, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("PlaylistInsert", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("AfterId", AfterId);
    request.WriteStringParameter("Name", Name);
    request.WriteStringParameter("Description", Description);
    request.WriteIntParameter("ImageId", ImageId);
    request.Send(function(result){
		result["NewId"] = request.ReadIntParameter(result["NewId"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylistManager.prototype.PlaylistDeleteId = function(Value, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("PlaylistDeleteId", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("Value", Value);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylistManager.prototype.PlaylistMove = function(Id, AfterId, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("PlaylistMove", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("Id", Id);
    request.WriteIntParameter("AfterId", AfterId);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylistManager.prototype.PlaylistsMax = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("PlaylistsMax", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Value"] = request.ReadIntParameter(result["Value"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylistManager.prototype.TracksMax = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("TracksMax", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Value"] = request.ReadIntParameter(result["Value"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylistManager.prototype.PlaylistArrays = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("PlaylistArrays", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Token"] = request.ReadIntParameter(result["Token"]);	
		result["IdArray"] = request.ReadBinaryParameter(result["IdArray"]);	
		result["TokenArray"] = request.ReadBinaryParameter(result["TokenArray"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylistManager.prototype.PlaylistArraysChanged = function(Token, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("PlaylistArraysChanged", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("Token", Token);
    request.Send(function(result){
		result["Value"] = request.ReadBoolParameter(result["Value"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylistManager.prototype.Read = function(Id, TrackId, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Read", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("Id", Id);
    request.WriteIntParameter("TrackId", TrackId);
    request.Send(function(result){
		result["Metadata"] = request.ReadStringParameter(result["Metadata"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylistManager.prototype.ReadList = function(Id, TrackIdList, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("ReadList", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("Id", Id);
    request.WriteStringParameter("TrackIdList", TrackIdList);
    request.Send(function(result){
		result["TrackList"] = request.ReadStringParameter(result["TrackList"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylistManager.prototype.Insert = function(Id, AfterTrackId, Metadata, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Insert", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("Id", Id);
    request.WriteIntParameter("AfterTrackId", AfterTrackId);
    request.WriteStringParameter("Metadata", Metadata);
    request.Send(function(result){
		result["NewTrackId"] = request.ReadIntParameter(result["NewTrackId"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylistManager.prototype.DeleteId = function(Id, TrackId, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("DeleteId", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("Id", Id);
    request.WriteIntParameter("TrackId", TrackId);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServicePlaylistManager.prototype.DeleteAll = function(Id, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("DeleteAll", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("Id", Id);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

