
var ServiceProduct = function(aId){	
	this.iUrl = window.location.protocol + "//" + window.location.host + "/" + aId + "/Product/control";
	this.iDomain = "av.openhome.org";
	if (this.iDomain == "upnp.org") {
		this.iDomain = "schemas.upnp.org";
    }
	this.iDomain = this.iDomain.replace(/\./,"-");
	this.iType = "Product";
	this.iVersion = "1";
	
	this.iVariables = {};
		this.iVariables["ManufacturerName"] = new ServiceVariable("ManufacturerName");
		this.iVariables["ManufacturerInfo"] = new ServiceVariable("ManufacturerInfo");
		this.iVariables["ManufacturerUrl"] = new ServiceVariable("ManufacturerUrl");
		this.iVariables["ManufacturerImageUri"] = new ServiceVariable("ManufacturerImageUri");
		this.iVariables["ModelName"] = new ServiceVariable("ModelName");
		this.iVariables["ModelInfo"] = new ServiceVariable("ModelInfo");
		this.iVariables["ModelUrl"] = new ServiceVariable("ModelUrl");
		this.iVariables["ModelImageUri"] = new ServiceVariable("ModelImageUri");
		this.iVariables["ProductRoom"] = new ServiceVariable("ProductRoom");
		this.iVariables["ProductName"] = new ServiceVariable("ProductName");
		this.iVariables["ProductInfo"] = new ServiceVariable("ProductInfo");
		this.iVariables["ProductUrl"] = new ServiceVariable("ProductUrl");
		this.iVariables["ProductImageUri"] = new ServiceVariable("ProductImageUri");
		this.iVariables["Standby"] = new ServiceVariable("Standby");
		this.iVariables["SourceIndex"] = new ServiceVariable("SourceIndex");
		this.iVariables["SourceCount"] = new ServiceVariable("SourceCount");
		this.iVariables["SourceXml"] = new ServiceVariable("SourceXml");
		this.iVariables["Attributes"] = new ServiceVariable("Attributes");
		this.iVariables["SourceXmlChangeCount"] = new ServiceVariable("SourceXmlChangeCount");
		this.iVariables["SourceType"] = new ServiceVariable("SourceType");
		this.iVariables["SourceName"] = new ServiceVariable("SourceName");
		this.iVariables["SourceVisible"] = new ServiceVariable("SourceVisible");
}


ServiceProduct.prototype.ServiceName = function(){
	return this.iType;
}

ServiceProduct.prototype.Variables = function(){
	return this.iVariables;
}

ServiceProduct.prototype.VariableNames = function(){
	var result = [];
	for (var variable in this.iVariables){
		if (this.iVariables.hasOwnProperty(variable)){
			result[result.length] = variable;
		}
	}
	return result;
}


ServiceProduct.prototype.Manufacturer = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Manufacturer", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Name"] = request.ReadStringParameter(result["Name"]);	
		result["Info"] = request.ReadStringParameter(result["Info"]);	
		result["Url"] = request.ReadStringParameter(result["Url"]);	
		result["ImageUri"] = request.ReadStringParameter(result["ImageUri"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceProduct.prototype.Model = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Model", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Name"] = request.ReadStringParameter(result["Name"]);	
		result["Info"] = request.ReadStringParameter(result["Info"]);	
		result["Url"] = request.ReadStringParameter(result["Url"]);	
		result["ImageUri"] = request.ReadStringParameter(result["ImageUri"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceProduct.prototype.Product = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Product", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Room"] = request.ReadStringParameter(result["Room"]);	
		result["Name"] = request.ReadStringParameter(result["Name"]);	
		result["Info"] = request.ReadStringParameter(result["Info"]);	
		result["Url"] = request.ReadStringParameter(result["Url"]);	
		result["ImageUri"] = request.ReadStringParameter(result["ImageUri"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceProduct.prototype.Standby = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Standby", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Value"] = request.ReadBoolParameter(result["Value"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceProduct.prototype.SetStandby = function(Value, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("SetStandby", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteBoolParameter("Value", Value);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceProduct.prototype.SourceCount = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("SourceCount", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Value"] = request.ReadIntParameter(result["Value"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceProduct.prototype.SourceXml = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("SourceXml", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Value"] = request.ReadStringParameter(result["Value"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceProduct.prototype.SourceIndex = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("SourceIndex", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Value"] = request.ReadIntParameter(result["Value"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceProduct.prototype.SetSourceIndex = function(Value, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("SetSourceIndex", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("Value", Value);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceProduct.prototype.SetSourceIndexByName = function(Value, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("SetSourceIndexByName", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteStringParameter("Value", Value);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceProduct.prototype.Source = function(Index, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Source", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("Index", Index);
    request.Send(function(result){
		result["SystemName"] = request.ReadStringParameter(result["SystemName"]);	
		result["Type"] = request.ReadStringParameter(result["Type"]);	
		result["Name"] = request.ReadStringParameter(result["Name"]);	
		result["Visible"] = request.ReadBoolParameter(result["Visible"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceProduct.prototype.Attributes = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Attributes", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Value"] = request.ReadStringParameter(result["Value"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceProduct.prototype.SourceXmlChangeCount = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("SourceXmlChangeCount", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Value"] = request.ReadIntParameter(result["Value"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

