
var ServiceReceiver = function(aId){	
	this.iUrl = window.location.protocol + "//" + window.location.host + "/" + aId + "/Receiver/control";
	this.iDomain = "av.openhome.org";
	if (this.iDomain == "upnp.org") {
		this.iDomain = "schemas.upnp.org";
    }
	this.iDomain = this.iDomain.replace(/\./,"-");
	this.iType = "Receiver";
	this.iVersion = "1";
	
	this.iVariables = {};
		this.iVariables["Uri"] = new ServiceVariable("Uri");
		this.iVariables["Metadata"] = new ServiceVariable("Metadata");
		this.iVariables["TransportState"] = new ServiceVariable("TransportState");
		this.iVariables["ProtocolInfo"] = new ServiceVariable("ProtocolInfo");
}

ServiceReceiver.kTransportStateStopped = "Stopped";
ServiceReceiver.kTransportStatePlaying = "Playing";
ServiceReceiver.kTransportStateWaiting = "Waiting";
ServiceReceiver.kTransportStateBuffering = "Buffering";

ServiceReceiver.prototype.ServiceName = function(){
	return this.iType;
}

ServiceReceiver.prototype.Variables = function(){
	return this.iVariables;
}

ServiceReceiver.prototype.VariableNames = function(){
	var result = [];
	for (var variable in this.iVariables){
		if (this.iVariables.hasOwnProperty(variable)){
			result[result.length] = variable;
		}
	}
	return result;
}


ServiceReceiver.prototype.Play = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Play", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceReceiver.prototype.Stop = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Stop", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceReceiver.prototype.SetSender = function(Uri, Metadata, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("SetSender", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteStringParameter("Uri", Uri);
    request.WriteStringParameter("Metadata", Metadata);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceReceiver.prototype.Sender = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Sender", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Uri"] = request.ReadStringParameter(result["Uri"]);	
		result["Metadata"] = request.ReadStringParameter(result["Metadata"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceReceiver.prototype.ProtocolInfo = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("ProtocolInfo", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Value"] = request.ReadStringParameter(result["Value"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceReceiver.prototype.TransportState = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("TransportState", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Value"] = request.ReadStringParameter(result["Value"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

