
var ServiceRenderingControl = function(aId){	
	this.iUrl = window.location.protocol + "//" + window.location.host + "/" + aId + "/RenderingControl/control";
	this.iDomain = "upnp.org";
	if (this.iDomain == "upnp.org") {
		this.iDomain = "schemas.upnp.org";
    }
	this.iDomain = this.iDomain.replace(/\./,"-");
	this.iType = "RenderingControl";
	this.iVersion = "1";
	
	this.iVariables = {};
		this.iVariables["PresetNameList"] = new ServiceVariable("PresetNameList");
		this.iVariables["Mute"] = new ServiceVariable("Mute");
		this.iVariables["Volume"] = new ServiceVariable("Volume");
		this.iVariables["VolumeDB"] = new ServiceVariable("VolumeDB");
		this.iVariables["LastChange"] = new ServiceVariable("LastChange");
		this.iVariables["Channel"] = new ServiceVariable("Channel");
		this.iVariables["InstanceID"] = new ServiceVariable("InstanceID");
		this.iVariables["PresetName"] = new ServiceVariable("PresetName");
}

ServiceRenderingControl.kChannelMaster = "Master";
ServiceRenderingControl.kPresetNameFactoryDefaults = "FactoryDefaults";

ServiceRenderingControl.prototype.ServiceName = function(){
	return this.iType;
}

ServiceRenderingControl.prototype.Variables = function(){
	return this.iVariables;
}

ServiceRenderingControl.prototype.VariableNames = function(){
	var result = [];
	for (var variable in this.iVariables){
		if (this.iVariables.hasOwnProperty(variable)){
			result[result.length] = variable;
		}
	}
	return result;
}


ServiceRenderingControl.prototype.ListPresets = function(InstanceID, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("ListPresets", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("InstanceID", InstanceID);
    request.Send(function(result){
		result["CurrentPresetNameList"] = request.ReadStringParameter(result["CurrentPresetNameList"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceRenderingControl.prototype.SelectPreset = function(InstanceID, PresetName, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("SelectPreset", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("InstanceID", InstanceID);
    request.WriteStringParameter("PresetName", PresetName);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceRenderingControl.prototype.GetMute = function(InstanceID, Channel, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("GetMute", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("InstanceID", InstanceID);
    request.WriteStringParameter("Channel", Channel);
    request.Send(function(result){
		result["CurrentMute"] = request.ReadBoolParameter(result["CurrentMute"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceRenderingControl.prototype.SetMute = function(InstanceID, Channel, DesiredMute, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("SetMute", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("InstanceID", InstanceID);
    request.WriteStringParameter("Channel", Channel);
    request.WriteBoolParameter("DesiredMute", DesiredMute);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceRenderingControl.prototype.GetVolume = function(InstanceID, Channel, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("GetVolume", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("InstanceID", InstanceID);
    request.WriteStringParameter("Channel", Channel);
    request.Send(function(result){
		result["CurrentVolume"] = request.ReadIntParameter(result["CurrentVolume"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceRenderingControl.prototype.SetVolume = function(InstanceID, Channel, DesiredVolume, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("SetVolume", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("InstanceID", InstanceID);
    request.WriteStringParameter("Channel", Channel);
    request.WriteIntParameter("DesiredVolume", DesiredVolume);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceRenderingControl.prototype.GetVolumeDB = function(InstanceID, Channel, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("GetVolumeDB", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("InstanceID", InstanceID);
    request.WriteStringParameter("Channel", Channel);
    request.Send(function(result){
		result["CurrentVolume"] = request.ReadIntParameter(result["CurrentVolume"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceRenderingControl.prototype.SetVolumeDB = function(InstanceID, Channel, DesiredVolume, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("SetVolumeDB", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("InstanceID", InstanceID);
    request.WriteStringParameter("Channel", Channel);
    request.WriteIntParameter("DesiredVolume", DesiredVolume);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceRenderingControl.prototype.GetVolumeDBRange = function(InstanceID, Channel, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("GetVolumeDBRange", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteIntParameter("InstanceID", InstanceID);
    request.WriteStringParameter("Channel", Channel);
    request.Send(function(result){
		result["MinValue"] = request.ReadIntParameter(result["MinValue"]);	
		result["MaxValue"] = request.ReadIntParameter(result["MaxValue"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

