
var ServiceSoundcard = function(aId){	
	this.iUrl = window.location.protocol + "//" + window.location.host + "/" + aId + "/Soundcard/control";
	this.iDomain = "linn.co.uk";
	if (this.iDomain == "upnp.org") {
		this.iDomain = "schemas.upnp.org";
    }
	this.iDomain = this.iDomain.replace(/\./,"-");
	this.iType = "Soundcard";
	this.iVersion = "1";
	
	this.iVariables = {};
		this.iVariables["ChannelUri"] = new ServiceVariable("ChannelUri");
		this.iVariables["ChannelMetadata"] = new ServiceVariable("ChannelMetadata");
		this.iVariables["TransportState"] = new ServiceVariable("TransportState");
		this.iVariables["ProtocolInfo"] = new ServiceVariable("ProtocolInfo");
}

ServiceSoundcard.kTransportStateStopped = "Stopped";
ServiceSoundcard.kTransportStatePlaying = "Playing";
ServiceSoundcard.kTransportStateBuffering = "Buffering";

ServiceSoundcard.prototype.ServiceName = function(){
	return this.iType;
}

ServiceSoundcard.prototype.Variables = function(){
	return this.iVariables;
}

ServiceSoundcard.prototype.VariableNames = function(){
	var result = [];
	for (var variable in this.iVariables){
		if (this.iVariables.hasOwnProperty(variable)){
			result[result.length] = variable;
		}
	}
	return result;
}


ServiceSoundcard.prototype.SetChannel = function(Uri, Metadata, aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("SetChannel", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.WriteStringParameter("Uri", Uri);
    request.WriteStringParameter("Metadata", Metadata);
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceSoundcard.prototype.Stop = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Stop", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceSoundcard.prototype.Play = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Play", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceSoundcard.prototype.Channel = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("Channel", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Uri"] = request.ReadStringParameter(result["Uri"]);	
		result["Metadata"] = request.ReadStringParameter(result["Metadata"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceSoundcard.prototype.ProtocolInfo = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("ProtocolInfo", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["Info"] = request.ReadStringParameter(result["Info"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

ServiceSoundcard.prototype.TransportState = function(aSuccessFunction, aErrorFunction){	
	var request = new SoapRequest("TransportState", this.iUrl, this.iDomain, this.iType, this.iVersion);		
    request.Send(function(result){
		result["State"] = request.ReadStringParameter(result["State"]);	
	
		if (aSuccessFunction){
			aSuccessFunction(result);
		}
	}, function(message, transport) {
		if (aErrorFunction) {aErrorFunction(message, transport);}
	});
}
    

